@@ -529,13 +529,14 @@ static int CmdDetectReader(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hw detectreader",
                   "Start to detect presences of reader field",
+                  "hw detectreader\n"
                   "hw detectreader -L\n"
                  );
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("L", "LF", "detect low frequency 125/134 kHz"),
-        arg_lit0("H", "HF", "detect high frequency 13.56 MHZ"),
+        arg_lit0("L", "LF", "only detect low frequency 125/134 kHz"),
+        arg_lit0("H", "HF", "only detect high frequency 13.56 MHZ"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -543,20 +544,35 @@ static int CmdDetectReader(const char *Cmd) {
     bool hf = arg_get_lit(ctx, 2);
     CLIParserFree(ctx);
 
-    if ((lf + hf) > 1) {
-        PrintAndLogEx(INFO, "Can only set one frequency");
-        return PM3_EINVARG;
+    // 0: Detect both frequency in mode 1
+    // 1: LF_ONLY
+    // 2: HF_ONLY
+    uint8_t arg = 0;
+    if (lf == true && hf == false) {
+        arg = 1;
+    } else if (hf == true && lf == false) {
+        arg = 2;
     }
 
-    uint8_t arg = 0;
-    if (lf)
-        arg = 1;
-    else if (hf)
-        arg = 2;
-
-    PrintAndLogEx(INFO, "press pm3 button to change modes and finally exit");
     clearCommandBuffer();
     SendCommandNG(CMD_LISTEN_READER_FIELD, (uint8_t *)&arg, sizeof(arg));
+    PrintAndLogEx(INFO, "Press " _GREEN_("pm3 button") " or " _GREEN_("<Enter>") " to change modes and exit");
+
+    for (;;) {
+        if (kbd_enter_pressed()) {
+            SendCommandNG(CMD_BREAK_LOOP, NULL, 0);
+            PrintAndLogEx(DEBUG, _GREEN_("<Enter>") " pressed");
+        }
+
+        PacketResponseNG resp;
+        if (WaitForResponseTimeout(CMD_LISTEN_READER_FIELD, &resp, 1000)) {
+            if (resp.status != PM3_EOPABORTED) {
+                PrintAndLogEx(ERR, "Unexpected response: %d", resp.status);
+            }
+            break;
+        }
+    }
+    PrintAndLogEx(INFO, "Done!");
     return PM3_SUCCESS;
 }
 
@@ -633,20 +649,62 @@ static int CmdLCDReset(const char *Cmd) {
 static int CmdReadmem(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hw readmem",
-                  "Read memory at decimal address from ARM chip flash.",
-                  "hw readmem -a 10000"
+                  "Reads processor flash memory into a file or views on console",
+                  "hw readmem -f myfile                    -> save 512KB processor flash memory to file\n"
+                  "hw readmem -a 8192 -l 512               -> display 512 bytes from offset 8192\n"
                  );
 
     void *argtable[] = {
         arg_param_begin,
-        arg_u64_1("a", "adr", "<dec>", "address to read"),
+        arg_u64_0("a", "adr", "<dec>", "flash address to start reading from"),
+        arg_u64_0("l", "len", "<dec>", "length (default 32 or 512KB)"),
+        arg_str0("f", "file", "<fn>", "save to file"),
+        arg_u64_0("c", "cols", "<dec>", "column breaks"),
+        arg_lit0("r", "raw", "use raw address mode: read from anywhere, not just flash"),
         arg_param_end
     };
-    CLIExecWithReturn(ctx, Cmd, argtable, true);
-    uint32_t address = arg_get_u32(ctx, 1);
+    CLIExecWithReturn(ctx, Cmd, argtable, false);
+
+    // check for -file option first to determine the output mode
+    int fnlen = 0;
+    char filename[FILE_PATH_SIZE] = {0};
+    CLIParamStrToBuf(arg_get_str(ctx, 3), (uint8_t *)filename, FILE_PATH_SIZE, &fnlen);
+    bool save_to_file = fnlen > 0;
+
+    // default len to 512KB when saving to file, to 32 bytes when viewing on the console.
+    uint32_t default_len = save_to_file ? 512 * 1024 : 32;
+
+    uint32_t address = arg_get_u32_def(ctx, 1, 0);
+    uint32_t len = arg_get_u32_def(ctx, 2, default_len);
+    int breaks = arg_get_int_def(ctx, 4, 32);
+    bool raw = arg_get_lit(ctx, 5);
     CLIParserFree(ctx);
-    clearCommandBuffer();
-    SendCommandNG(CMD_READ_MEM, (uint8_t *)&address, sizeof(address));
+
+    uint8_t *buffer = calloc(len, sizeof(uint8_t));
+    if (!buffer) {
+        PrintAndLogEx(ERR, "error, cannot allocate memory ");
+        return PM3_EMALLOC;
+    }
+
+    const char *flash_str = raw ? "" : " flash";
+    PrintAndLogEx(INFO, "reading "_YELLOW_("%u")" bytes from processor%s memory",
+                  len, flash_str);
+
+    DeviceMemType_t type = raw ? MCU_MEM : MCU_FLASH;
+    if (!GetFromDevice(type, buffer, len, address, NULL, 0, NULL, -1, true)) {
+        PrintAndLogEx(FAILED, "ERROR; reading from MCU flash memory");
+        free(buffer);
+        return PM3_EFLASH;
+    }
+
+    if (save_to_file) {
+        saveFile(filename, ".bin", buffer, len);
+    } else {
+        PrintAndLogEx(INFO, "---- " _CYAN_("processor%s memory") " ----", flash_str);
+        print_hex_break(buffer, len, breaks);
+    }
+
+    free(buffer);
     return PM3_SUCCESS;
 }
 
@@ -701,6 +759,45 @@ static int CmdSetDivisor(const char *Cmd) {
     return PM3_SUCCESS;
 }
 
+static int CmdSetHFThreshold(const char *Cmd) {
+
+    CLIParserContext *ctx;
+    CLIParserInit(&ctx, "hw sethfthresh",
+                  "Set thresholds in HF/14a and Legic mode.",
+                  "hw sethfthresh -t 7 -i 20 -l 8"
+                 );
+
+    void *argtable[] = {
+        arg_param_begin,
+        arg_int0("t", "thresh", "<dec>", "threshold, used in 14a reader mode (def 7)"),
+        arg_int0("i", "high", "<dec>", "high threshold, used in 14a sniff mode (def 20)"),
+        arg_int0("l", "legic", "<dec>", "threshold used in Legic mode (def 8)"),
+        arg_param_end
+    };
+    CLIExecWithReturn(ctx, Cmd, argtable, true);
+
+    struct {
+        uint8_t threshold;
+        uint8_t threshold_high;
+        uint8_t legic_threshold;
+    } PACKED params;
+
+    params.threshold = arg_get_int_def(ctx, 1, 7);
+    params.threshold_high = arg_get_int_def(ctx, 2, 20);
+    params.legic_threshold = arg_get_int_def(ctx, 3, 8);
+    CLIParserFree(ctx);
+
+    if ((params.threshold < 1) || (params.threshold > 63) || (params.threshold_high < 1) || (params.threshold_high > 63)) {
+        PrintAndLogEx(ERR, "Thresholds must be between " _YELLOW_("1") " and " _YELLOW_("63"));
+        return PM3_EINVARG;
+    }
+
+    clearCommandBuffer();
+    SendCommandNG(CMD_HF_ISO14443A_SET_THRESHOLDS, (uint8_t *)&params, sizeof(params));
+    PrintAndLogEx(SUCCESS, "Thresholds set.");
+    return PM3_SUCCESS;
+}
+
 static int CmdSetMux(const char *Cmd) {
 
     CLIParserContext *ctx;
@@ -1150,9 +1247,10 @@ static command_t CommandTable[] = {
     {"lcd",           CmdLCD,          IfPm3Lcd,        "Send command/data to LCD"},
     {"lcdreset",      CmdLCDReset,     IfPm3Lcd,        "Hardware reset LCD"},
     {"ping",          CmdPing,         IfPm3Present,    "Test if the Proxmark3 is responsive"},
-    {"readmem",       CmdReadmem,      IfPm3Present,    "Read memory at decimal address from flash"},
+    {"readmem",       CmdReadmem,      IfPm3Present,    "Read from processor flash"},
     {"reset",         CmdReset,        IfPm3Present,    "Reset the Proxmark3"},
     {"setlfdivisor",  CmdSetDivisor,   IfPm3Present,    "Drive LF antenna at 12MHz / (divisor + 1)"},
+    {"sethfthresh",   CmdSetHFThreshold, IfPm3Present,   "Set thresholds in HF/14a mode"},
     {"setmux",        CmdSetMux,       IfPm3Present,    "Set the ADC mux to a specific value"},
     {"standalone",    CmdStandalone,   IfPm3Present,    "Jump to the standalone mode"},
     {"status",        CmdStatus,       IfPm3Present,    "Show runtime status information about the connected Proxmark3"},
