@@ -104,7 +104,7 @@ static void em410x_construct_emul_graph(uint8_t *uid, uint8_t clock, uint8_t gap
     AppendGraph(true, clock, 0);
 }
 
-//print 64 bit EM410x ID in multiple formats
+// print 64 bit EM410x ID in multiple formats
 void printEM410x(uint32_t hi, uint64_t id, bool verbose, int type) {
 
     if (!id && !hi) return;
@@ -234,6 +234,39 @@ void printEM410x(uint32_t hi, uint64_t id, bool verbose, int type) {
         PrintAndLogEx(INFO, "------------------------------------------------");
     }
 }
+
+// takes 1s and 0s and searches for EM410x format - output EM ID
+static int ask_em410x_binary_decode(bool verbose, uint32_t *hi, uint64_t *lo, uint8_t *bits, size_t *size, size_t *idx) {
+
+    int ans = Em410xDecode(bits, size, idx, hi, lo);
+    if (ans < 0) {
+        if (ans == -2)
+            PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x not enough samples after demod");
+        else if (ans == -4)
+            PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x preamble not found");
+        else if (ans == -5)
+            PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x Size not correct: %zu", size);
+        else if (ans == -6)
+            PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x parity failed");
+
+        return PM3_ESOFT;
+    }
+
+    if (!lo && !hi) {
+        PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x decoded to all zeros");
+        return PM3_ESOFT;
+    }
+
+    PrintAndLogEx(DEBUG, "DEBUG: Em410x idx: %zu, Len: %zu, Printing DemodBuffer:", *idx, *size);
+    if (g_debugMode) {
+        printDemodBuff(0, false, false, true);
+    }
+
+    printEM410x(*hi, *lo, verbose, ans);
+    gs_em410xid = *lo;
+    return PM3_SUCCESS;
+}
+
 /* Read the ID of an EM410x tag.
  * Format:
  *   1111 1111 1           <-- standard non-repeatable header
@@ -246,42 +279,19 @@ int AskEm410xDecode(bool verbose, uint32_t *hi, uint64_t *lo) {
     size_t idx = 0;
     uint8_t bits[512] = {0};
     size_t size = sizeof(bits);
-    if (!getDemodBuff(bits, &size)) {
+    if (getDemodBuff(bits, &size) == false) {
         PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x problem during copy from ASK demod");
         return PM3_ESOFT;
     }
 
-    int ans = Em410xDecode(bits, &size, &idx, hi, lo);
-    if (ans < 0) {
+    int ret = ask_em410x_binary_decode(verbose, hi, lo, bits, &size, &idx);
 
-        if (ans == -2)
-            PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x not enough samples after demod");
-        else if (ans == -4)
-            PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x preamble not found");
-        else if (ans == -5)
-            PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x Size not correct: %zu", size);
-        else if (ans == -6)
-            PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x parity failed");
-
-        return PM3_ESOFT;
+    if (ret == PM3_SUCCESS) {
+        // set g_GraphBuffer for clone or sim command
+        setDemodBuff(g_DemodBuffer, (size == 40) ? 64 : 128, idx + 1);
+        setClockGrid(g_DemodClock, g_DemodStartIdx + ((idx + 1)*g_DemodClock));
     }
-    if (!lo && !hi) {
-        PrintAndLogEx(DEBUG, "DEBUG: Error - Em410x decoded to all zeros");
-        return PM3_ESOFT;
-    }
-
-    //set g_GraphBuffer for clone or sim command
-    setDemodBuff(g_DemodBuffer, (size == 40) ? 64 : 128, idx + 1);
-    setClockGrid(g_DemodClock, g_DemodStartIdx + ((idx + 1)*g_DemodClock));
-
-    PrintAndLogEx(DEBUG, "DEBUG: Em410x idx: %zu, Len: %zu, Printing DemodBuffer:", idx, size);
-    if (g_debugMode) {
-        printDemodBuff(0, false, false, true);
-    }
-
-    printEM410x(*hi, *lo, verbose, ans);
-    gs_em410xid = *lo;
-    return PM3_SUCCESS;
+    return ret;
 }
 
 int AskEm410xDemod(int clk, int invert, int maxErr, size_t maxLen, bool amplify, uint32_t *hi, uint64_t *lo, bool verbose) {
@@ -291,8 +301,9 @@ int AskEm410xDemod(int clk, int invert, int maxErr, size_t maxLen, bool amplify,
     if (isGraphBitstream()) {
         convertGraphFromBitstream();
     }
-    if (ASKDemod_ext(clk, invert, maxErr, maxLen, amplify, false, false, 1, &st) != PM3_SUCCESS)
+    if (ASKDemod_ext(clk, invert, maxErr, maxLen, amplify, false, false, 1, &st) != PM3_SUCCESS) {
         return PM3_ESOFT;
+    }
     return AskEm410xDecode(verbose, hi, lo);
 }
 
@@ -319,7 +330,7 @@ static int CmdEM410xWatch(const char *Cmd) {
     return lfsim_wait_check(CMD_LF_EM410X_WATCH);
 }
 
-//by marshmellow
+// by marshmellow
 //takes 3 arguments - clock, invert and maxErr as integers
 //attempts to demodulate ask while decoding manchester
 //prints binary found and saves in graphbuffer for further commands
@@ -348,6 +359,7 @@ static int CmdEM410xDemod(const char *Cmd) {
         arg_u64_0(NULL, "len", "<dec>", "maximum length"),
         arg_lit0("i", "invert", "invert output"),
         arg_lit0("a", "amp", "amplify signal"),
+        arg_str0(NULL, "bin", "<bin>", "Binary string i.e 0001001001"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -357,12 +369,26 @@ static int CmdEM410xDemod(const char *Cmd) {
     size_t max_len = arg_get_u32_def(ctx, 3, 0);
     bool invert = arg_get_lit(ctx, 4);
     bool amplify = arg_get_lit(ctx, 5);
+    int bin_len = 512;
+    uint8_t bin[512] = {0};
+    CLIGetStrWithReturn(ctx, 6, bin, &bin_len);
     CLIParserFree(ctx);
 
     uint32_t hi = 0;
     uint64_t lo = 0;
-    if (AskEm410xDemod(clk, invert, max_err, max_len, amplify, &hi, &lo, true) != PM3_SUCCESS)
+
+    if (bin_len) {
+        // 111111111011110010010100001110010101001101110001100100000010010011111100001111
+        size_t demodlen = bin_len;
+        size_t start_idx = 0;
+        uint8_t arr[258];
+        binstr_2_binarray(arr, (char *)bin, bin_len);
+        return ask_em410x_binary_decode(true, &hi, &lo, arr, &demodlen, &start_idx);
+    }
+
+    if (AskEm410xDemod(clk, invert, max_err, max_len, amplify, &hi, &lo, true) != PM3_SUCCESS) {
         return PM3_ESOFT;
+    }
 
     return PM3_SUCCESS;
 }
@@ -645,6 +671,7 @@ static int CmdEM410xClone(const char *Cmd) {
         arg_str1(NULL, "id", "<hex>", "EM Tag ID number (5 hex bytes)"),
         arg_lit0(NULL, "q5", "optional - specify writing to Q5/T5555 tag"),
         arg_lit0(NULL, "em", "optional - specify writing to EM4305/4469 tag"),
+        arg_lit0(NULL, "electra", "optional - add Electra blocks to tag"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, false);
@@ -656,13 +683,10 @@ static int CmdEM410xClone(const char *Cmd) {
     CLIGetHexWithReturn(ctx, 2, uid, &uid_len);
     bool q5 = arg_get_lit(ctx, 3);
     bool em = arg_get_lit(ctx, 4);
+    bool add_electra = arg_get_lit(ctx, 5);
     CLIParserFree(ctx);
 
     uint64_t id = bytes_to_num(uid, uid_len);
-    if (id == 0) {
-        PrintAndLogEx(ERR, "Cardnumber can't be zero");
-        return PM3_EINVARG;
-    }
 
     if (q5 && em) {
         PrintAndLogEx(FAILED, "Can't specify both Q5 and EM4305 at the same time");
@@ -680,6 +704,7 @@ static int CmdEM410xClone(const char *Cmd) {
     struct {
         bool Q5;
         bool EM;
+        bool add_electra;
         uint8_t clock;
         uint32_t high;
         uint32_t low;
@@ -687,6 +712,7 @@ static int CmdEM410xClone(const char *Cmd) {
 
     payload.Q5 = q5;
     payload.EM = em;
+    payload.add_electra = add_electra;
     payload.clock = clk;
     payload.high = (uint32_t)(id >> 32);
     payload.low = (uint32_t)id;
