@@ -471,7 +471,7 @@ static const fdxbCountryMapping_t fdxbCountryMapping[] = {
     { 985, "HomeAgain (Destron Fearing/Digital Angel)" },
     { 991, "Peeva" },
     { 999, "Test range" },
-    { 0,   "N/A" } // must be the last entry
+    { 0,   "n/a" } // must be the last entry
 };
 
 static const char *mapFDBX(uint16_t countryCode) {
@@ -878,10 +878,10 @@ static int CmdFdxBSim(const char *Cmd) {
     PacketResponseNG resp;
     WaitForResponse(CMD_LF_ASK_SIMULATE, &resp);
 
-    PrintAndLogEx(INFO, "Done");
-    if (resp.status != PM3_EOPABORTED)
+    PrintAndLogEx(INFO, "Done!");
+    if (resp.status != PM3_EOPABORTED) {
         return resp.status;
-
+    }
     return PM3_SUCCESS;
 }
 
