@@ -380,10 +380,10 @@ static int CmdPacSim(const char *Cmd) {
     PacketResponseNG resp;
     WaitForResponse(CMD_LF_NRZ_SIMULATE, &resp);
 
-    PrintAndLogEx(INFO, "Done");
-    if (resp.status != PM3_EOPABORTED)
+    PrintAndLogEx(INFO, "Done!");
+    if (resp.status != PM3_EOPABORTED) {
         return resp.status;
-
+    }
     return PM3_SUCCESS;
 }
 
