@@ -218,14 +218,20 @@ int MifareAuth4(mf4Session_t *mf4session, uint8_t *keyn, uint8_t *key, bool acti
 
     uint8_t cmd1[] = {0x70, keyn[1], keyn[0], 0x00};
     int res = ExchangeRAW14a(cmd1, sizeof(cmd1), activateField, true, data, sizeof(data), &datalen, silentMode);
-    if (res) {
-        if (!silentMode) PrintAndLogEx(ERR, "Exchange raw error: %d", res);
-        if (dropFieldIfError) DropField();
+    if (res != PM3_SUCCESS) {
+        if (silentMode == false) {
+            PrintAndLogEx(ERR, "Exchange raw error: %d", res);
+        }
+
+        if (dropFieldIfError) {
+            DropField();
+        }
         return PM3_ERFTRANS;
     }
 
-    if (verbose)
-        PrintAndLogEx(INFO, "<phase1: %s", sprint_hex(data, datalen));
+    if (verbose) {
+        PrintAndLogEx(INFO, "< phase1: %s", sprint_hex(data, datalen));
+    }
 
     if (datalen < 1) {
         if (!silentMode) PrintAndLogEx(ERR, "Card response wrong length: %d", datalen);
@@ -247,8 +253,9 @@ int MifareAuth4(mf4Session_t *mf4session, uint8_t *keyn, uint8_t *key, bool acti
 
     aes_decode(NULL, key, &data[1], RndB, 16);
     RndB[16] = RndB[0];
-    if (verbose)
+    if (verbose) {
         PrintAndLogEx(INFO, "RndB: %s", sprint_hex(RndB, 16));
+    }
 
     uint8_t cmd2[33] = {0};
     cmd2[0] = 0x72;
@@ -262,14 +269,19 @@ int MifareAuth4(mf4Session_t *mf4session, uint8_t *keyn, uint8_t *key, bool acti
         PrintAndLogEx(INFO, ">phase2: %s", sprint_hex(cmd2, 33));
     }
     res = ExchangeRAW14a(cmd2, sizeof(cmd2), false, true, data, sizeof(data), &datalen, silentMode);
-    if (res) {
-        if (!silentMode) PrintAndLogEx(ERR, "Exchange raw error: %d", res);
-        if (dropFieldIfError) DropField();
+    if (res != PM3_SUCCESS) {
+        if (silentMode == false) {
+            PrintAndLogEx(ERR, "Exchange raw error: %d", res);
+        }
+        if (dropFieldIfError) {
+            DropField();
+        }
         return PM3_ERFTRANS;
     }
 
-    if (verbose)
-        PrintAndLogEx(INFO, "<phase2: %s", sprint_hex(data, datalen));
+    if (verbose) {
+        PrintAndLogEx(INFO, "< phase2: %s", sprint_hex(data, datalen));
+    }
 
     aes_decode(NULL, key, &data[1], raw, 32);
 
@@ -320,11 +332,13 @@ int MifareAuth4(mf4Session_t *mf4session, uint8_t *keyn, uint8_t *key, bool acti
         PrintAndLogEx(INFO, "kmac: %s", sprint_hex(kmac, 16));
     }
 
-    if (!leaveSignalON)
+    if (leaveSignalON == false) {
         DropField();
+    }
 
-    if (verbose)
+    if (verbose) {
         PrintAndLogEx(NORMAL, "");
+    }
 
     if (mf4session) {
         mf4session->Authenticated = true;
@@ -341,21 +355,23 @@ int MifareAuth4(mf4Session_t *mf4session, uint8_t *keyn, uint8_t *key, bool acti
         memmove(mf4session->Kmac, kmac, 16);
     }
 
-    if (verbose)
+    if (verbose) {
         PrintAndLogEx(INFO, "Authentication OK");
+    }
 
     return PM3_SUCCESS;
 }
 
 static int intExchangeRAW14aPlus(uint8_t *datain, int datainlen, bool activateField, bool leaveSignalON, uint8_t *dataout, int maxdataoutlen, int *dataoutlen) {
-    if (g_verbose_mode)
+    if (g_verbose_mode) {
         PrintAndLogEx(INFO, ">>> %s", sprint_hex(datain, datainlen));
+    }
 
     int res = ExchangeRAW14a(datain, datainlen, activateField, leaveSignalON, dataout, maxdataoutlen, dataoutlen, false);
 
-    if (g_verbose_mode)
+    if (g_verbose_mode) {
         PrintAndLogEx(INFO, "<<< %s", sprint_hex(dataout, *dataoutlen));
-
+    }
     return res;
 }
 
@@ -373,48 +389,79 @@ int MFPCommitPerso(bool activateField, bool leaveSignalON, uint8_t *dataout, int
 }
 
 int MFPReadBlock(mf4Session_t *mf4session, bool plain, bool nomaccmd, bool nomacres, uint8_t blockNum, uint8_t blockCount, bool activateField, bool leaveSignalON, uint8_t *dataout, int maxdataoutlen, int *dataoutlen, uint8_t *mac) {
+
     int cmdb = 0x31;
-    if (nomacres) {cmdb = cmdb ^ 0x01;} // If we do not want MAC in reply, remove 0x01
-    if (plain) {cmdb = cmdb ^ 0x02;} // If we do not need an encrypted transmission, add 0x02
-    if (nomaccmd) {cmdb = cmdb ^ 0x04;} // If we do not want to send a MAC, remove 0x04
+    if (nomacres) {
+        cmdb = cmdb ^ 0x01;  // If we do not want MAC in reply, remove 0x01
+    }
+
+    if (plain) {
+        cmdb = cmdb ^ 0x02;  // If we do not need an encrypted transmission, add 0x02
+    }
+
+    if (nomaccmd) {
+        cmdb = cmdb ^ 0x04; // If we do not want to send a MAC, remove 0x04
+    }
+
     uint8_t rcmd1[4] = {cmdb, blockNum, 0x00, blockCount};
     uint8_t maccmddat[8] = {0};
     uint8_t rcmd[nomaccmd ? 4 : 12];
-    if (!nomaccmd && mf4session)
-        CalculateMAC(mf4session, mtypReadCmd, blockNum, blockCount, rcmd1, 4, &maccmddat[0], g_verbose_mode);
-    memmove(rcmd, rcmd1, 4);
-    if (!nomaccmd) {memmove(&rcmd[4], maccmddat, 8);}
-    int res = intExchangeRAW14aPlus(rcmd, sizeof(rcmd), activateField, leaveSignalON, dataout, maxdataoutlen, dataoutlen);
-    if (res)
-        return res;
-    if (mf4session)
-        mf4session->R_Ctr++;
-    if (mf4session && !nomacres && *dataoutlen > 11)
-        CalculateMAC(mf4session, mtypReadResp, blockNum, blockCount, dataout, *dataoutlen - 8 - 2, mac, g_verbose_mode);
 
-    return 0;
+    if (nomaccmd == false && mf4session) {
+        CalculateMAC(mf4session, mtypReadCmd, blockNum, blockCount, rcmd1, 4, &maccmddat[0], g_verbose_mode);
+    }
+
+    memmove(rcmd, rcmd1, 4);
+    if (nomaccmd == false) {
+        memmove(&rcmd[4], maccmddat, 8);
+    }
+
+    int res = intExchangeRAW14aPlus(rcmd, sizeof(rcmd), activateField, leaveSignalON, dataout, maxdataoutlen, dataoutlen);
+    if (res != PM3_SUCCESS) {
+        return res;
+    }
+
+    if (mf4session) {
+        mf4session->R_Ctr++;
+    }
+
+    if (mf4session && !nomacres && *dataoutlen > 11) {
+        CalculateMAC(mf4session, mtypReadResp, blockNum, blockCount, dataout, *dataoutlen - 8 - 2, mac, g_verbose_mode);
+    }
+
+    return PM3_SUCCESS;
 }
 
 int MFPWriteBlock(mf4Session_t *mf4session, bool plain, bool nomacres, uint8_t blockNum, uint8_t blockHdr, uint8_t *data, bool activateField, bool leaveSignalON, uint8_t *dataout, int maxdataoutlen, int *dataoutlen, uint8_t *mac) {
     int cmdb = 0xA1;
-    if (nomacres) {cmdb = cmdb ^ 0x01;} // If we do not want MAC in reply, remove 0x01
-    if (plain) {cmdb = cmdb ^ 0x02;} // If we do not need an encrypted transmission, add 0x02
+    if (nomacres) {
+        cmdb = cmdb ^ 0x01; // If we do not want MAC in reply, remove 0x01
+    }
+
+    if (plain) {
+        cmdb = cmdb ^ 0x02; // If we do not need an encrypted transmission, add 0x02
+    }
+
     uint8_t rcmd[1 + 2 + 16 + 8] = {cmdb, blockNum, blockHdr};
     memmove(&rcmd[3], data, 16);
-    if (mf4session)
+    if (mf4session) {
         CalculateMAC(mf4session, mtypWriteCmd, blockNum, 1, rcmd, 19, &rcmd[19], g_verbose_mode);
+    }
 
     int res = intExchangeRAW14aPlus(rcmd, sizeof(rcmd), activateField, leaveSignalON, dataout, maxdataoutlen, dataoutlen);
-    if (res)
+    if (res != PM3_SUCCESS) {
         return res;
+    }
 
-    if (mf4session)
+    if (mf4session) {
         mf4session->W_Ctr++;
+    }
 
-    if (mf4session && mac && *dataoutlen > 3 && !nomacres)
+    if (mf4session && mac && *dataoutlen > 3 && !nomacres) {
         CalculateMAC(mf4session, mtypWriteResp, blockNum, 1, dataout, *dataoutlen, mac, g_verbose_mode);
+    }
 
-    return 0;
+    return PM3_SUCCESS;
 }
 
 int mfpReadSector(uint8_t sectorNo, uint8_t keyType, uint8_t *key, uint8_t *dataout, bool verbose) {
@@ -424,8 +471,9 @@ int mfpReadSector(uint8_t sectorNo, uint8_t keyType, uint8_t *key, uint8_t *data
     uint16_t uKeyNum = 0x4000 + sectorNo * 2 + (keyType ? 1 : 0);
     keyn[0] = uKeyNum >> 8;
     keyn[1] = uKeyNum & 0xff;
-    if (verbose)
+    if (verbose) {
         PrintAndLogEx(INFO, "--sector[%u]:%02x key:%04x", mfNumBlocksPerSector(sectorNo), sectorNo, uKeyNum);
+    }
 
     mf4Session_t _session;
     int res = MifareAuth4(&_session, keyn, key, true, true, true, verbose, false);
@@ -467,16 +515,18 @@ int mfpReadSector(uint8_t sectorNo, uint8_t keyType, uint8_t *key, uint8_t *data
             PrintAndLogEx(WARNING, "MAC   card: %s", sprint_hex(&data[1 + 16], 8));
             PrintAndLogEx(WARNING, "MAC reader: %s", sprint_hex(mac, 8));
 
-            if (!verbose)
+            if (verbose == false) {
                 return 7;
+            }
         } else {
-            if (verbose)
+            if (verbose) {
                 PrintAndLogEx(INFO, "MAC: %s", sprint_hex(&data[1 + 16], 8));
+            }
         }
     }
     DropField();
 
-    return 0;
+    return PM3_SUCCESS;
 }
 
 int MFPGetSignature(bool activateField, bool leaveSignalON, uint8_t *dataout, int maxdataoutlen, int *dataoutlen) {
@@ -501,13 +551,13 @@ int MFPGetVersion(bool activateField, bool leaveSignalON, uint8_t *dataout, int
     if (tmp[0] == 0xAF) {
         c[0] = 0xAF;
         res = intExchangeRAW14aPlus(c, sizeof(c), false, true, tmp, maxdataoutlen, dataoutlen);
-        if (res == 0) {
+        if (res == PM3_SUCCESS) {
 
             memcpy(dataout + 7, tmp + 1, (*dataoutlen - 3));
 
             // MFDES_ADDITIONAL_FRAME
             res = intExchangeRAW14aPlus(c, sizeof(c), false, false, tmp, maxdataoutlen, dataoutlen);
-            if (res == 0) {
+            if (res == PM3_SUCCESS) {
                 if (tmp[0] == 0x90) {
                     memcpy(dataout + 7 + 7, tmp + 1, (*dataoutlen - 3));
                     *dataoutlen = 28;
