@@ -109,6 +109,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "data bitsamples" },
     { 1, "data bmap" },
     { 1, "data clear" },
+    { 1, "data crypto" },
     { 1, "data diff" },
     { 0, "data hexsamples" },
     { 1, "data hex2bin" },
@@ -157,18 +158,19 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf 14a ndefread" },
     { 0, "hf 14a ndefwrite" },
     { 1, "hf 14b help" },
+    { 1, "hf 14b list" },
     { 0, "hf 14b apdu" },
     { 0, "hf 14b dump" },
     { 0, "hf 14b info" },
-    { 1, "hf 14b list" },
     { 0, "hf 14b ndefread" },
     { 0, "hf 14b raw" },
+    { 0, "hf 14b rdbl" },
     { 0, "hf 14b reader" },
     { 0, "hf 14b sim" },
     { 0, "hf 14b sniff" },
-    { 0, "hf 14b rdbl" },
-    { 0, "hf 14b sriwrite" },
+    { 0, "hf 14b wrbl" },
     { 1, "hf 14b view" },
+    { 1, "hf 14b valid" },
     { 1, "hf 15 help" },
     { 1, "hf 15 list" },
     { 1, "hf 15 demod" },
@@ -182,6 +184,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf 15 restore" },
     { 0, "hf 15 samples" },
     { 1, "hf 15 view" },
+    { 0, "hf 15 wipe" },
     { 0, "hf 15 wrbl" },
     { 0, "hf 15 sim" },
     { 0, "hf 15 eload" },
@@ -223,11 +226,11 @@ const static vocabulary_t vocabulary[] = {
     { 1, "hf emrtd list" },
     { 1, "hf felica help" },
     { 1, "hf felica list" },
-    { 0, "hf felica reader" },
     { 0, "hf felica info" },
-    { 0, "hf felica sniff" },
     { 0, "hf felica raw" },
     { 0, "hf felica rdbl" },
+    { 0, "hf felica reader" },
+    { 0, "hf felica sniff" },
     { 0, "hf felica wrbl" },
     { 0, "hf felica rqservice" },
     { 0, "hf felica rqresponse" },
@@ -258,17 +261,6 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf gallagher delete" },
     { 1, "hf gallagher diversifykey" },
     { 1, "hf gallagher decode" },
-    { 1, "hf ksx6924 help" },
-    { 0, "hf ksx6924 select" },
-    { 0, "hf ksx6924 info" },
-    { 0, "hf ksx6924 balance" },
-    { 0, "hf ksx6924 init" },
-    { 0, "hf ksx6924 prec" },
-    { 1, "hf jooki help" },
-    { 0, "hf jooki clone" },
-    { 1, "hf jooki decode" },
-    { 1, "hf jooki encode" },
-    { 0, "hf jooki sim" },
     { 1, "hf iclass help" },
     { 1, "hf iclass list" },
     { 0, "hf iclass dump" },
@@ -296,6 +288,22 @@ const static vocabulary_t vocabulary[] = {
     { 1, "hf iclass managekeys" },
     { 1, "hf iclass permutekey" },
     { 0, "hf iclass sam" },
+    { 1, "hf ict help" },
+    { 0, "hf ict credential" },
+    { 0, "hf ict info" },
+    { 1, "hf ict list" },
+    { 1, "hf ict reader" },
+    { 1, "hf jooki help" },
+    { 0, "hf jooki clone" },
+    { 1, "hf jooki decode" },
+    { 1, "hf jooki encode" },
+    { 0, "hf jooki sim" },
+    { 1, "hf ksx6924 help" },
+    { 0, "hf ksx6924 select" },
+    { 0, "hf ksx6924 info" },
+    { 0, "hf ksx6924 balance" },
+    { 0, "hf ksx6924 init" },
+    { 0, "hf ksx6924 prec" },
     { 1, "hf legic help" },
     { 0, "hf legic dump" },
     { 0, "hf legic info" },
@@ -327,6 +335,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf mf nested" },
     { 1, "hf mf hardnested" },
     { 0, "hf mf staticnested" },
+    { 0, "hf mf brute" },
     { 0, "hf mf autopwn" },
     { 0, "hf mf nack" },
     { 0, "hf mf chk" },
@@ -376,6 +385,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf mf gchpwd" },
     { 0, "hf mf gdmcfg" },
     { 0, "hf mf gdmsetcfg" },
+    { 1, "hf mf gdmparsecfg" },
     { 0, "hf mf gdmsetblk" },
     { 0, "hf mf ndefformat" },
     { 0, "hf mf ndefread" },
@@ -419,18 +429,19 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf mfu sim" },
     { 0, "hf mfu setpwd" },
     { 0, "hf mfu setuid" },
+    { 0, "hf mfu amiibo" },
     { 1, "hf mfdes help" },
-    { 0, "hf mfdes info" },
-    { 0, "hf mfdes getuid" },
-    { 0, "hf mfdes default" },
+    { 1, "hf mfdes list" },
     { 0, "hf mfdes auth" },
     { 0, "hf mfdes chk" },
+    { 0, "hf mfdes default" },
     { 0, "hf mfdes detect" },
-    { 0, "hf mfdes freemem" },
-    { 0, "hf mfdes setconfig" },
     { 0, "hf mfdes formatpicc" },
-    { 1, "hf mfdes list" },
+    { 0, "hf mfdes freemem" },
+    { 0, "hf mfdes getuid" },
+    { 0, "hf mfdes info" },
     { 0, "hf mfdes mad" },
+    { 0, "hf mfdes setconfig" },
     { 0, "hf mfdes lsapp" },
     { 0, "hf mfdes getaids" },
     { 0, "hf mfdes getappnames" },
@@ -502,11 +513,14 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf vas reader" },
     { 1, "hf vas decrypt" },
     { 1, "hf waveshare help" },
-    { 0, "hf waveshare loadbmp" },
+    { 1, "hf waveshare load" },
     { 1, "hf xerox help" },
+    { 1, "hf xerox list" },
     { 0, "hf xerox info" },
-    { 0, "hf xerox reader" },
     { 0, "hf xerox dump" },
+    { 0, "hf xerox reader" },
+    { 1, "hf xerox view" },
+    { 0, "hf xerox rdbl" },
     { 1, "hw help" },
     { 0, "hw break" },
     { 0, "hw bootloader" },
@@ -520,6 +534,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hw readmem" },
     { 0, "hw reset" },
     { 0, "hw setlfdivisor" },
+    { 0, "hw sethfthresh" },
     { 0, "hw setmux" },
     { 0, "hw standalone" },
     { 0, "hw status" },
@@ -567,12 +582,14 @@ const static vocabulary_t vocabulary[] = {
     { 1, "lf em 4x05 help" },
     { 0, "lf em 4x05 brute" },
     { 0, "lf em 4x05 chk" },
+    { 1, "lf em 4x05 config" },
     { 1, "lf em 4x05 demod" },
     { 0, "lf em 4x05 dump" },
     { 0, "lf em 4x05 info" },
     { 0, "lf em 4x05 read" },
     { 1, "lf em 4x05 sniff" },
     { 0, "lf em 4x05 unlock" },
+    { 1, "lf em 4x05 view" },
     { 0, "lf em 4x05 wipe" },
     { 0, "lf em 4x05 write" },
     { 1, "lf em 4x50 help" },
@@ -584,9 +601,10 @@ const static vocabulary_t vocabulary[] = {
     { 0, "lf em 4x50 rdbl" },
     { 0, "lf em 4x50 reader" },
     { 0, "lf em 4x50 restore" },
+    { 1, "lf em 4x50 view" },
+    { 0, "lf em 4x50 wipe" },
     { 0, "lf em 4x50 wrbl" },
     { 0, "lf em 4x50 wrpwd" },
-    { 0, "lf em 4x50 wipe" },
     { 0, "lf em 4x50 eload" },
     { 0, "lf em 4x50 esave" },
     { 0, "lf em 4x50 eview" },
@@ -626,11 +644,13 @@ const static vocabulary_t vocabulary[] = {
     { 0, "lf hitag info" },
     { 0, "lf hitag dump" },
     { 0, "lf hitag read" },
+    { 1, "lf hitag view" },
     { 0, "lf hitag wrbl" },
     { 0, "lf hitag sniff" },
     { 0, "lf hitag cc" },
     { 0, "lf hitag ta" },
     { 0, "lf hitag eload" },
+    { 0, "lf hitag eview" },
     { 0, "lf hitag sim" },
     { 1, "lf idteck help" },
     { 1, "lf idteck demod" },
@@ -794,13 +814,13 @@ const static vocabulary_t vocabulary[] = {
     { 1, "piv list" },
     { 1, "smart help" },
     { 1, "smart list" },
+    { 0, "smart brute" },
     { 0, "smart info" },
-    { 0, "smart relay" },
+    { 1, "smart pcsc" },
     { 0, "smart reader" },
     { 0, "smart raw" },
     { 1, "smart upgrade" },
     { 0, "smart setclock" },
-    { 0, "smart brute" },
     { 1, "script help" },
     { 1, "script list" },
     { 1, "script run" },
@@ -829,4 +849,4 @@ const static vocabulary_t vocabulary[] = {
 }
 #endif
 
-#endif
\ No newline at end of file
+#endif
