@@ -119,7 +119,7 @@ int searchHomeFilePath(char **foundpath, const char *subdir, const char *filenam
         pathlen += strlen(subdir);
         char *tmp = realloc(path, pathlen * sizeof(char));
         if (tmp == NULL) {
-            //free(path);
+            free(path);
             return PM3_EMALLOC;
         }
         path = tmp;
@@ -156,7 +156,7 @@ int searchHomeFilePath(char **foundpath, const char *subdir, const char *filenam
     pathlen += strlen(filename);
     char *tmp = realloc(path, pathlen * sizeof(char));
     if (tmp == NULL) {
-        //free(path);
+        free(path);
         return PM3_EMALLOC;
     }
 
@@ -646,8 +646,9 @@ void iceSimple_Filter(int *data, const size_t len, uint8_t k) {
     }
 }
 
-void print_progress(size_t count, uint64_t max, barMode_t style) {
+void print_progress(uint64_t count, uint64_t max, barMode_t style) {
     int cols = 100 + 35;
+    max = (count > max) ? count : max;
 #if defined(HAVE_READLINE)
     static int prev_cols = 0;
     int rows;
@@ -698,13 +699,15 @@ void print_progress(size_t count, uint64_t max, barMode_t style) {
     for (; i < unit * value; i += unit) {
         memcpy(bar + i, block[mode], unit);
     }
-    // add last block
-    if (mode == 1) {
-        memcpy(bar + i, smoothtable[PERCENTAGEFRAC(count, max)], unit);
-    } else {
-        memcpy(bar + i, space[mode], unit);
+    if (i < unit * width) {
+        // add last block
+        if (mode == 1) {
+            memcpy(bar + i, smoothtable[PERCENTAGEFRAC(count, max)], unit);
+        } else {
+            memcpy(bar + i, space[mode], unit);
+        }
+        i += unit;
     }
-    i += unit;
     // add spaces
     for (; i < unit * width; i += unit) {
         memcpy(bar + i, space[mode], unit);
@@ -730,11 +733,11 @@ void print_progress(size_t count, uint64_t max, barMode_t style) {
             break;
         }
         case STYLE_MIXED: {
-            printf("\b%c[2K\r[" _YELLOW_("=")"] %s [ %zu mV / %2u V / %2u Vmax ]", 27, cbar, count, (uint32_t)(count / 1000), (uint32_t)(max / 1000));
+            printf("\b%c[2K\r[" _YELLOW_("=")"] %s [ %"PRIu64" mV / %2u V / %2u Vmax ]", 27, cbar, count, (uint32_t)(count / 1000), (uint32_t)(max / 1000));
             break;
         }
         case STYLE_VALUE: {
-            printf("[" _YELLOW_("=")"] %zu mV / %2u V / %2u Vmax   \r", count, (uint32_t)(count / 1000), (uint32_t)(max / 1000));
+            printf("[" _YELLOW_("=")"] %"PRIu64" mV / %2u V / %2u Vmax   \r", count, (uint32_t)(count / 1000), (uint32_t)(max / 1000));
             break;
         }
     }
